@@ -26,6 +26,7 @@
 #include <locale.h>
 #include <fcntl.h>
 #include <sys/wait.h>
+#include <grp.h>
 #include <pwd.h>
 
 #include <glib.h>
@@ -35,10 +36,6 @@
 
 #include "gdm-common.h"
 
-#ifndef HAVE_MKDTEMP
-#include "mkdtemp.h"
-#endif
-
 #include <systemd/sd-login.h>
 
 #define GDM_DBUS_NAME                            "org.gnome.DisplayManager"
@@ -47,12 +44,6 @@
 
 G_DEFINE_QUARK (gdm-common-error, gdm_common_error);
 
-const char *
-gdm_make_temp_dir (char *template)
-{
-        return mkdtemp (template);
-}
-
 gboolean
 gdm_clear_close_on_exec_flag (int fd)
 {
@@ -97,6 +88,24 @@ gdm_get_pwent_for_name (const char     *name,
         return (pwent != NULL);
 }
 
+static gboolean
+gdm_get_grent_for_gid (gint           gid,
+                       struct group **grentp)
+{
+        struct group *grent;
+
+        do {
+                errno = 0;
+                grent = getgrgid (gid);
+        } while (grent == NULL && errno == EINTR);
+
+        if (grentp != NULL) {
+                *grentp = grent;
+        }
+
+        return (grent != NULL);
+}
+
 int
 gdm_wait_on_and_disown_pid (int pid,
                             int timeout)
@@ -343,10 +352,10 @@ create_transient_display (GDBusConnection *connection,
         return TRUE;
 }
 
-static gboolean
-activate_session_id (GDBusConnection *connection,
-                     const char      *seat_id,
-                     const char      *session_id)
+gboolean
+gdm_activate_session_by_id (GDBusConnection *connection,
+                            const char      *seat_id,
+                            const char      *session_id)
 {
         GError *local_error = NULL;
         GVariant *reply;
@@ -372,13 +381,14 @@ activate_session_id (GDBusConnection *connection,
         return TRUE;
 }
 
-static gboolean
-get_login_window_session_id (const char  *seat_id,
-                             char       **session_id)
+gboolean
+gdm_get_login_window_session_id (const char  *seat_id,
+		                 char       **session_id)
 {
         gboolean   ret;
         int        res, i;
         char     **sessions;
+        char      *service_id;
         char      *service_class;
         char      *state;
 
@@ -390,13 +400,19 @@ get_login_window_session_id (const char  *seat_id,
 
         if (sessions == NULL || sessions[0] == NULL) {
                 *session_id = NULL;
-                ret = TRUE;
+                ret = FALSE;
                 goto out;
         }
 
         for (i = 0; sessions[i]; i ++) {
+
                 res = sd_session_get_class (sessions[i], &service_class);
                 if (res < 0) {
+                        if (res == -ENOENT) {
+                                free (service_class);
+                                continue;
+                        }
+
                         g_debug ("failed to determine class of session %s: %s", sessions[i], strerror (-res));
                         ret = FALSE;
                         goto out;
@@ -422,21 +438,35 @@ get_login_window_session_id (const char  *seat_id,
                 }
                 free (state);
 
-                *session_id = g_strdup (sessions[i]);
-                ret = TRUE;
-                break;
+                res = sd_session_get_service (sessions[i], &service_id);
+                if (res < 0) {
+                        g_debug ("failed to determine service of session %s: %s", sessions[i], strerror (-res));
+                        ret = FALSE;
+                        goto out;
+                }
 
+                if (strcmp (service_id, "gdm-launch-environment") == 0) {
+                        *session_id = g_strdup (sessions[i]);
+                        ret = TRUE;
+
+                        free (service_id);
+                        goto out;
+                }
+
+                free (service_id);
         }
 
         *session_id = NULL;
-        ret = TRUE;
+        ret = FALSE;
 
 out:
-        for (i = 0; sessions[i]; i ++) {
-                free (sessions[i]);
-        }
+        if (sessions) {
+                for (i = 0; sessions[i]; i ++) {
+                        free (sessions[i]);
+                }
 
-        free (sessions);
+                free (sessions);
+        }
 
         return ret;
 }
@@ -497,9 +527,9 @@ goto_login_session (GDBusConnection  *connection,
                 return FALSE;
         }
 
-        res = get_login_window_session_id (seat_id, &session_id);
+        res = gdm_get_login_window_session_id (seat_id, &session_id);
         if (res && session_id != NULL) {
-                res = activate_session_id (connection, seat_id, session_id);
+                res = gdm_activate_session_by_id (connection, seat_id, session_id);
 
                 if (res) {
                         ret = TRUE;
@@ -543,7 +573,7 @@ listify_hash (const char *key,
 {
         char *str;
         str = g_strdup_printf ("%s=%s", key, value);
-        g_debug ("GdmSlave: script environment: %s", str);
+        g_debug ("Gdm: script environment: %s", str);
         g_ptr_array_add (env, str);
 }
 
@@ -586,6 +616,13 @@ gdm_get_script_environment (const char *username,
 
                         g_hash_table_insert (hash, g_strdup ("SHELL"),
                                              g_strdup (pwent->pw_shell));
+
+                        /* Also get group name and propagate down */
+                        struct group *grent;
+
+                        if (gdm_get_grent_for_gid (pwent->pw_gid, &grent)) {
+                                g_hash_table_insert (hash, g_strdup ("GROUP"), g_strdup (grent->gr_name));
+                        }
                 }
         }
 
@@ -594,8 +631,13 @@ gdm_get_script_environment (const char *username,
         }
 
         /* Runs as root */
-        g_hash_table_insert (hash, g_strdup ("XAUTHORITY"), g_strdup (display_x11_authority_file));
-        g_hash_table_insert (hash, g_strdup ("DISPLAY"), g_strdup (display_name));
+        if (display_x11_authority_file) {
+                g_hash_table_insert (hash, g_strdup ("XAUTHORITY"), g_strdup (display_x11_authority_file));
+        }
+
+        if (display_name) {
+                g_hash_table_insert (hash, g_strdup ("DISPLAY"), g_strdup (display_name));
+        }
         g_hash_table_insert (hash, g_strdup ("PATH"), g_strdup (GDM_SESSION_DEFAULT_PATH));
         g_hash_table_insert (hash, g_strdup ("RUNNING_UNDER_GDM"), g_strdup ("true"));
 
